#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "graficos.h"
#include "resource.h"

/*  Declare Windows procedure  */
LRESULT CALLBACK WindowProcedure (HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DialogProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DialogoOpciones(HWND, UINT, WPARAM, LPARAM);
void ProcesarScrollV(HWND hDlg, HWND Control, int Codigo, int Posicion);
void LeerRegistro(Graficos *graficos);
void GuardarRegistro(Graficos *graficos);

/*  Make the class name into a global variable  */
char szClassName[ ] = "SolitarioApp";

int WINAPI WinMain (HINSTANCE hThisInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR lpszArgument,
                     int nFunsterStil)
{
    HWND hwnd;               /* This is the handle for our window */
    MSG messages;            /* Here messages to the application are saved */
    WNDCLASSEX wincl;        /* Data structure for the windowclass */
    HACCEL hAcelerador;      /* Manupulador de aceleradores */

    /* The Window structure */
    wincl.hInstance = hThisInstance;
    wincl.lpszClassName = szClassName;
    wincl.lpfnWndProc = WindowProcedure;      /* This function is called by windows */
    wincl.style = CS_DBLCLKS;                 /* Catch double-clicks */
    wincl.cbSize = sizeof (WNDCLASSEX);

    /* Use default icon and mouse-pointer */
    wincl.hIcon = LoadIcon (hThisInstance, "Icono");
    wincl.hIconSm = LoadIcon (hThisInstance, "Icono");
    wincl.hCursor = LoadCursor (NULL, IDC_ARROW);
    wincl.lpszMenuName = "Menu";               /* No menu */
    wincl.cbClsExtra = 0;                      /* No extra bytes after the window class */
    wincl.cbWndExtra = 0;                      /* structure or the window instance */
    /* Use Windows's default color as the background of the window */
    wincl.hbrBackground = CreateSolidBrush(RGB(0,127,0));

    /* Register the window class, and if it fails quit the program */
    if (!RegisterClassEx (&wincl))
        return 0;

    /* The class is registered, let's create the program*/
    hwnd = CreateWindowEx (
           0,                   /* Extended possibilites for variation */
           szClassName,         /* Classname */
           "Solitario Con Clase",/* Title Text */
           WS_OVERLAPPEDWINDOW, /* default window */
           CW_USEDEFAULT,       /* Windows decides the position */
           CW_USEDEFAULT,       /* where the window ends up on the screen */
           500,                 /* The programs width */
           500,                 /* and height in pixels */
           HWND_DESKTOP,        /* The window is a child-window to desktop */
           NULL,                /* No menu */
           hThisInstance,       /* Program Instance handler */
           NULL                 /* No Window Creation data */
           );

    /* Make the window visible on the screen */
    ShowWindow (hwnd, nFunsterStil);

    hAcelerador = LoadAccelerators(hThisInstance, "Aceleradores");

    /* Ejecutar el bucle de mensajes hasta que GetMessage() retorno con un
    valor distinto de TRUE (-1 para error, o FALSE)  */
    while (TRUE == GetMessage (&messages, NULL, 0, 0))
    {
        if(!TranslateAccelerator(hwnd, hAcelerador, &messages)) {
           /* Traducir mensajes de teclas virtuales a mensajes de caracteres
              slo si TranslateAccelerator regresa con nulo */
           TranslateMessage(&messages);
           /* Enviar el mensaje a WindowProcedure */
           DispatchMessage(&messages);
        }
    }

    /* The program return-value is 0 - The value that PostQuitMessage() gave */
    return messages.wParam;
}


/*  This function is called by the Windows function DispatchMessage()  */
LRESULT CALLBACK WindowProcedure (HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    static Tapete tapete;
    static Graficos *graficos;
    static HINSTANCE hInstance;
    PAINTSTRUCT ps;
    int zona, destino;
    Movimiento mov;
    POINT punto;
    char titulo[64], msg[128];

    switch (message)                  /* handle the messages */
    {
        case WM_CREATE:
            hInstance = ((LPCREATESTRUCT)lParam)->hInstance;
            srand(time(0));
            graficos = new Graficos(hInstance, hwnd, tapete);
            LeerRegistro(graficos);
            MoveWindow(hwnd, 30, 30, 500, 500, FALSE);
            tapete.Iniciar();
            return false;
        case WM_LBUTTONDBLCLK:
            // La funcin ComienzaArrastre sirve para saber sobre qu
            // pila se encuentra el cursor:
            zona = graficos->RatonEnZona(LOWORD(lParam), HIWORD(lParam));
            // Si no es la zona del MAZO, verificar si se puede extraer la carta:
            if(zona != -1 && zona != MAZO) {
                destino = tapete.VerificarDobleClick(zona);
                if(destino != -1) {
                    tapete.MoverNaipe(zona, destino);
                    graficos->MostrarMovimiento(zona, destino);
                    if(tapete.Terminado()) {
                        LoadString(hInstance, ID_TITULO, titulo, 64);
                        LoadString(hInstance, ID_RESUELTO, msg, 128);
                        MessageBox(hwnd, msg, titulo, MB_OK);
                    } else if(tapete.SinSolucion()) {
                        LoadString(hInstance, ID_TITULO, titulo, 64);
                        LoadString(hInstance, ID_SINSOLUCION, msg, 128);
                        MessageBox(hwnd, msg, titulo, MB_OK);
                    }
                }
            }
            break;
        case WM_LBUTTONDOWN:
            zona = graficos->ComienzaArrastre(LOWORD(lParam), HIWORD(lParam));
            if(zona == MAZO) {
                tapete.MoverNaipe(MAZO, MONTON);
                graficos->MostrarMovimiento(MAZO, MONTON);
                if(tapete.SinSolucion()) {
                    LoadString(hInstance, ID_TITULO, titulo, 64);
                    LoadString(hInstance, ID_SINSOLUCION, msg, 128);
                    MessageBox(hwnd, msg, titulo, MB_OK);
                }
            } else if(zona > MAZO && tapete.CogeNaipe(zona)) {
                SetCursor(graficos->Mover());
                SetCapture(hwnd);
                graficos->IniciaCaptura(zona);
            }
            break;
        case WM_MOUSEMOVE:
            if(tapete.EnArrastre()) {
                graficos->EnArrastre(LOWORD(lParam), HIWORD(lParam));
            }
            break;
        case WM_LBUTTONUP:
            if(tapete.EnArrastre()) {
                zona = graficos->FinArrastre(LOWORD(lParam), HIWORD(lParam));
                ReleaseCapture();
                // Si el movimiento NO es legal, devolver naipe a origen:
                if(zona == -1 || !tapete.MovimientoLegal(zona)) {
                    zona = tapete.Origen();
                }
                tapete.SueltaNaipe(zona);
                graficos->MostrarMovimiento(-1, zona);
                SetCursor(graficos->Flecha());
                if(tapete.Terminado()) {
                    LoadString(hInstance, ID_TITULO, titulo, 64);
                    LoadString(hInstance, ID_RESUELTO, msg, 128);
                    MessageBox(hwnd, msg, titulo, MB_OK);
                } else if(tapete.SinSolucion()) {
                    LoadString(hInstance, ID_TITULO, titulo, 64);
                    LoadString(hInstance, ID_SINSOLUCION, msg, 128);
                    MessageBox(hwnd, msg, titulo, MB_OK);
                }
            }
            break;
        case WM_PAINT:
            BeginPaint(hwnd, &ps);
            graficos->MostrarTapete();
            EndPaint(hwnd, &ps);
            break;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case CMD_NUEVO:
                    tapete.Limpiar();
                    tapete.Iniciar();
                    graficos->Iniciar();
                    InvalidateRect(hwnd, NULL, TRUE);
                    break;
                case CMD_AYUDA:
                    DialogBox(hInstance, "DlgAyuda", hwnd, DialogProc);
                    break;
                case CMD_DESHACER:
                    mov = tapete.DesHacer();
                    if(mov.Origen() != -1 )
                        graficos->MostrarMovimiento(mov.Destino(), mov.Origen());
                    break;
                case CMD_SOBRE:
                    DialogBox(hInstance, "DlgSobre", hwnd, DialogProc);
                    break;
                case CMD_OPCIONES:
                    DialogBoxParam(hInstance, "DlgOpciones", hwnd, DialogoOpciones, (LPARAM)graficos);
                    InvalidateRect(hwnd, NULL, TRUE);
                    break;
                case CMD_SALIR:
                    PostQuitMessage (0);
                    break;
            }
            break;
        case WM_DESTROY:
            PostQuitMessage (0);       /* send a WM_QUIT to the message queue */
            break;
        default:                      /* for messages that we don't deal with */
            return DefWindowProc (hwnd, message, wParam, lParam);
    }

    return 0;
}

BOOL CALLBACK DialogProc(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    switch (uMsg)                  /* manipulador del mensaje */
    {
        case WM_COMMAND:
           EndDialog(hDlg, FALSE);
           return TRUE;
    }
    return FALSE;
}

BOOL CALLBACK DialogoOpciones(HWND hDlg, UINT uMsg, WPARAM wParam, LPARAM lParam) {
    static Graficos *graficos;
    static int valor;
    SCROLLINFO siv = {
       sizeof(SCROLLINFO),
       SIF_POS | SIF_RANGE | SIF_PAGE,
       53, 69,
       5,
       0,
       0};
    PAINTSTRUCT ps;
    HDC hDC;
    HDC memDC;
    POINT punto[3];
    HBITMAP hDorso;

    switch (uMsg)                  /* manipulador del mensaje */
    {
        case WM_INITDIALOG:
            graficos = (Graficos*)lParam;
            siv.nPos = valor = graficos->Dorso();
            SetScrollInfo(GetDlgItem(hDlg, IDC_OPC_SCROLL), SB_CTL, &siv, TRUE);
            break;
        case WM_VSCROLL:
            ProcesarScrollV(hDlg, (HWND)lParam,
               (int)LOWORD(wParam), (int)HIWORD(wParam));
            valor = SendDlgItemMessage(hDlg, IDC_OPC_SCROLL, SBM_GETPOS, 0, 0);
            InvalidateRect(hDlg, NULL, TRUE);
            return FALSE;
        case WM_PAINT:
            hDC = BeginPaint(hDlg, &ps);
            hDorso = LoadBitmap(graficos->Lib(), MAKEINTRESOURCE(valor));
            memDC = CreateCompatibleDC(hDC);
            punto[0].x = 50;
            punto[0].y = 20;
            punto[1].x = punto[0].x+graficos->W();
            punto[1].y = punto[0].y;
            punto[2].x = punto[0].x;
            punto[2].y = punto[0].y+graficos->H();
            SelectObject(memDC, hDorso);
            PlgBlt(hDC, punto, memDC, 0, 0, graficos->W(), graficos->H(),
                graficos->Mascara(), 0, 0);
            DeleteObject(hDorso);
            DeleteDC(memDC);
            EndPaint(hDlg, &ps);
            break;
        case WM_COMMAND:
            switch(LOWORD(wParam)) {
                case IDOK:
                    graficos->CambiarDorso(valor);
                    GuardarRegistro(graficos);
                    EndDialog(hDlg, TRUE);
                    break;
                case IDCANCEL:
                    EndDialog(hDlg, FALSE);
                    break;
            }
            return TRUE;
    }
    return FALSE;
}

void ProcesarScrollV(HWND hDlg, HWND Control, int Codigo, int Posicion) {
   SCROLLINFO si = {
       sizeof(SCROLLINFO),
       SIF_ALL, 0, 0, 0, 0, 0};

   GetScrollInfo(Control, SB_CTL, &si);

   switch(Codigo) {
      case SB_BOTTOM:
         si.nPos = si.nMin;
         break;
      case SB_TOP:
         si.nPos = si.nMax;
         break;
      case SB_LINEDOWN:
         si.nPos++;
         break;
      case SB_LINEUP:
         si.nPos--;
         break;
      case SB_PAGEDOWN:
         si.nPos += si.nPage;
         break;
      case SB_PAGEUP:
         si.nPos -= si.nPage;
         break;
      case SB_THUMBPOSITION:
      case SB_THUMBTRACK:
         si.nPos = Posicion;
      case SB_ENDSCROLL:
         break;
   }
   if(si.nPos < si.nMin) si.nPos = si.nMin;
   if(si.nPos > si.nMax-si.nPage+1) si.nPos = si.nMax-si.nPage+1;

   SetScrollInfo(Control, SB_CTL, &si, true);
}

void LeerRegistro(Graficos *graficos) {
   HKEY clave;
   DWORD disposicion;
   DWORD tamanio=sizeof(int);
   int dorso = 64;

   // Crear o abrir la clave de registro:
   RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\ConClase\\Solitario",
       0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
       &clave, &disposicion);

   if(disposicion == REG_CREATED_NEW_KEY) { // Es necesario crear los valores
      RegSetValueEx(clave, "Dorso", 0, REG_DWORD,
         (BYTE *)&dorso, sizeof(int));
   }
   else { // Leer valores desde el registro:
      RegQueryValueEx(clave, "Dorso", NULL, NULL,
         (BYTE*)&dorso, &tamanio);
   }
   graficos->CambiarDorso(dorso);
   RegCloseKey(clave);
}

void GuardarRegistro(Graficos *graficos) {
   HKEY clave;
   DWORD disposicion;
   DWORD tamanio;
   int dorso = graficos->Dorso();

   // Crear o abrir la clave de registro:
   RegCreateKeyEx(HKEY_LOCAL_MACHINE, "SOFTWARE\\ConClase\\Solitario",
       0, "", REG_OPTION_NON_VOLATILE, KEY_ALL_ACCESS, NULL,
       &clave, &disposicion);

   RegSetValueEx(clave, "Dorso", 0, REG_DWORD,
      (BYTE *)&dorso, sizeof(DWORD));
   RegCloseKey(clave);
}
